<?php
/**
 * api新分销商合伙人Controller
 * Created by PhpStorm.
 * User: haoren
 * Date: 2020/07/22
 * Time: 15:00
 */

namespace JinDouYun\Controller\NewCommission;

use Exception;
use Mall\Framework\Core\ErrorCode;
use Mall\Framework\Core\ResultWrapper;
use Mall\Framework\Core\StatusCode;
use JinDouYun\Controller\BaseController;

use Util\Common\AgentTools;
use JinDouYun\Model\Customer\MCustomer;

class ApiNewAgent extends BaseController
{
    
    private $customerData;
    private $agentTool;

    public function __construct($isCheckAcl = false, $isMustLogin = true, $checkToken=true)
    {
        parent::__construct($isCheckAcl, $isMustLogin, $checkToken);
        $this->agentTool = new AgentTools($this->onlineEnterpriseId);
        $this->customerData = $this->agentTool->getCustomerDataByUserCenterId($this->onlineUserId);

    }
    
    
    /**
     * 获取账户信息
     */
    public function getApiAgentData(){

        $data = $this->agentTool->apiAgentInfoData($this->onlineUserId);
        if(empty($data) || $data["deleteStatus"]!=5){
            parent::sendOutput('数据不存在', ErrorCode::$paramError);
        }
        parent::sendOutput($data);
    }
    /**
     * 获取分佣明细
     */
    public function getApiAgentDetail(){
        $params = $this->request->getRawJson();
        if(empty($params)){
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $pageParams = pageToOffset($params['page'] ?: 1, $params['pageSize'] ?: 10);
        $selectParams['limit'] = $pageParams['limit'];
        $selectParams['offset'] = $pageParams['offset'];
        $where=[];
        if(!empty($params['code']) && in_array($params['code'], ["consume","income"])){
            if($params['code']=="consume"){
                $where["type"] = 0;
            }
            if($params['code']=="income"){
                $where["type"] = 1;
            }
        }
        $where["type"] = 0;
        $selectParams["where"] = $where;
        
        $data = $this->agentTool->getCommissionDetail($this->customerData["id"],$selectParams);
        parent::sendOutput($data);
    }
    /**
     * 获取分佣账户余额明细
     */
    public function getApiAgentBalanceDetail(){
        $params = $this->request->getRawJson();
        if(empty($params)){
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $pageParams = pageToOffset($params['page'] ?: 1, $params['pageSize'] ?: 10);
        $selectParams['limit'] = $pageParams['limit'];
        $selectParams['offset'] = $pageParams['offset'];
        $data = $this->agentTool->getCommissionBalanceDetail($this->customerData["id"],$selectParams);
        parent::sendOutput($data);
    }
    /**
     * 获取提现记录
     */
    public function getApiAgentApplyCashList(){
        $params = $this->request->getRawJson();
        if(empty($params)){
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $pageParams = pageToOffset($params['page'] ?: 1, $params['pageSize'] ?: 10);
        $selectParams['limit'] = $pageParams['limit'];
        $selectParams['offset'] = $pageParams['offset'];
        $where=[];
        if(!empty($params['status'])){
            if($params['status']=="wait"){
                $where["status"] = 0;
            }
            if($params['status']=="ok"){
                $where["status"] = 1;
            }
            if($params['status']=="no"){
                $where["status"] = -1;
            }
        }
        $selectParams["where"] = $where;
        $data = $this->agentTool->getPartnerApplyCashList($this->customerData["id"],$selectParams);
        parent::sendOutput($data);
    }
    /**
     * 获取提现详情
     */
    public function getApiAgentApplyCashInfo(){
        $params = $this->request->getRawJson();
        if(empty($params) || empty($params["id"])){
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        $where["id"] = $params["id"];
        $selectParams["where"] = $where;
        $data = $this->agentTool->getPartnerApplyCashInfo($this->customerData["id"],$selectParams);
        parent::sendOutput($data);
            
    }
    
    public function getApiAgentCashType(){
        $data = $this->agentTool->getBankType(1);
        parent::sendOutput($data);
        
    }
    
    public function apiAgentApplyCash(){
        $params = $this->request->getRawJson();
        if(empty($params)){
            parent::sendOutput('参数为空', ErrorCode::$paramError);
        }
        
        
        $data=[];
        $data["customerId"] = $this->customerData["id"];
        $data["money"]     = empty($params["money"])    ? 0  : floatval($params["money"]);
        $data["bank_type"] = empty($params["bank_type"])? "" : $params["bank_type"];
        $data["bank_name"] = empty($params["bank_name"])? "" : $params["bank_name"];
        $data["bank_num"]  = empty($params["bank_num"]) ? "" : $params["bank_num"];
        $data["bank_ad"]   = empty($params["bank_ad"])  ? "" : $params["bank_ad"];
 
        
        
        $res = $this->agentTool->applyCash($data);
        if(empty($res) || empty($res["code"])){
            parent::sendOutput('系统繁忙,请稍后重试001', ErrorCode::$paramError);
        }
        if($res["code"]==-1){
            parent::sendOutput($res["msg"], ErrorCode::$paramError);
        }
        parent::sendOutput("提现申请已提交,请耐心等待处理");
        
    }
    
    

    


}
